/* ISDN Monitor panel applet
 * Insprired by kimon by Markus Dahlweid dahlweid@tzi.org 
 *
 * By Uwe Steinmann <Uwe.Steinmann@fernuni-hagen.de>
 *
 * Enjoy.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License  
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sys/time.h>
#include <stdio.h>

#include <gnome.h>
#include <applet-widget.h>
#include "gimon.h"


#include "gimon_green.xpm"
#include "gimon_yellow.xpm"
#include "gimon_red.xpm"

char **line0_pixmap;
char **line1_pixmap;
GtkWidget *applet;
GtkWidget *button;
GtkWidget *counter;
GtkWidget *button_pixmap0;
GtkWidget *button_pixmap1;
GtkWidget *popup_window;
GtkWidget *process_clist;
GdkPixmap *green_pixmap; 
GdkPixmap *yellow_pixmap;
GdkPixmap *red_pixmap;
GtkTooltips *tooltip;
int updatetime;
gint timeout;
gint timer_list;
int popup_width;
int popup_height;
int popup_open;
char *none_in_filename;
GdkPixmap *none_in_pixmap;
GdkBitmap *none_in_pixmap_mask;
char *none_out_filename;
GdkPixmap *none_out_pixmap;
GdkBitmap *none_out_pixmap_mask;
char *modem_in_filename;
GdkPixmap *modem_in_pixmap;
GdkBitmap *modem_in_pixmap_mask;
char *modem_out_filename;
GdkPixmap *modem_out_pixmap;
GdkBitmap *modem_out_pixmap_mask;
char *net_in_filename;
GdkPixmap *net_in_pixmap;
GdkBitmap *net_in_pixmap_mask;
char *net_out_filename;
GdkPixmap *net_out_pixmap;
GdkBitmap *net_out_pixmap_mask;
char *raw_in_filename;
GdkPixmap *raw_in_pixmap;
GdkBitmap *raw_in_pixmap_mask;
char *raw_out_filename;
GdkPixmap *raw_out_pixmap;
GdkBitmap *raw_out_pixmap_mask;
char *voice_in_filename;
GdkPixmap *voice_in_pixmap;
GdkBitmap *voice_in_pixmap_mask;
char *voice_out_filename;
GdkPixmap *voice_out_pixmap;
GdkBitmap *voice_out_pixmap_mask;
char *fax_in_filename;
GdkPixmap *fax_in_pixmap;
GdkBitmap *fax_in_pixmap_mask;
char *fax_out_filename;
GdkPixmap *fax_out_pixmap;
GdkBitmap *fax_out_pixmap_mask;

extern void prefs_cb(GtkWidget *w, gpointer p);
extern void loadup_file(GdkPixmap **pixmap, GdkBitmap **mask, char *file);

static void redraw_pixmaps(void);
static ISDN_STATUS *get_isdn_status();

static void
about_cb(GtkWidget *w, gpointer d)
{
	gchar *authors[4];
	GtkWidget *about;
	
	authors[0] = "Uwe Steinmann";
	authors[1] = _("Insprired by kimon by Markus Dahlweid dahlweid@tzi.org");
	authors[2] = NULL;
	
	about = gnome_about_new (
		"GNOME ISDN Monitor",
		VERSION,
		"(C) 1999 Uwe Steinmann",
		(const gchar**)authors,
		_("This little applet just displays the status of the two ISDN lines."),
		NULL);
	
	gtk_widget_show(about);
	
	return;
}

static void
mount_cb(GtkWidget *w, gpointer d)
{
}

int
count_timer(GtkWidget *data)
{
	redraw_pixmaps();
	fprintf(stderr, "count timer: %d\n", updatetime);
	return(1);
}

gint run_command(GtkWidget *widget, gpointer data) {
	return 0;
}

static gint close_pop(GtkWidget *widget,gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(process_clist));
	process_clist=NULL;

	gdk_pointer_ungrab(GDK_CURRENT_TIME);
	gtk_grab_remove(GTK_WIDGET(popup_window));
	gtk_widget_destroy(GTK_WIDGET(popup_window));
	gtk_timeout_remove(timer_list);
	widget=NULL;	
	
	popup_open=FALSE;	
	return FALSE;

}

void set_proc_list(void)
{
	gint i;
	gchar *pcs[4];
	char text[200];
	ISDN_STATUS *isdn_status;

	isdn_status = get_isdn_status();
	for(i=0; i < ISDN_MAX_CHANNELS; i++) {
		if(isdn_status->drmap[i] >= 0) {
			pcs[0]=strdup(isdn_status->idmap[i]);
			pcs[1]=strdup(isdn_status->phone[i]);
			switch(isdn_status->usage[i] & ISDN_USAGE_MASK) {
				case 1:
					pcs[2]="Raw";
					break;
				case 2:
					pcs[2]="Modem";
					break;
				case 3:
					pcs[2]="Net";
					break;
				case 4:
					pcs[2]="Voice";
					break;
				case 5:
					pcs[2]="Fax";
					break;
				default:
					pcs[2]="None";
			}
			if(isdn_status->usage[i] & ISDN_USAGE_OUTGOING)
				pcs[3]="Out";
			else
				pcs[3]="In";
			gtk_clist_append(GTK_CLIST(process_clist),pcs);	
		}
	}
	free(isdn_status);
}

static gint scroll_click(GtkWidget *widget, gpointer data)
{
  return TRUE;
}

GtkWidget* create_scroll_win(GtkWidget *widget)
{
	GtkWidget *scrolled_window=NULL;
	char *nom_cols[4]={"Name","Phone","Status","Type"};
		
	scrolled_window=gtk_scrolled_window_new(NULL,NULL);
	
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
	     	                            GTK_POLICY_AUTOMATIC,
           	                        GTK_POLICY_AUTOMATIC);
    
	gtk_container_set_border_width(GTK_CONTAINER(scrolled_window),1);
 
	gtk_signal_connect(GTK_OBJECT(scrolled_window),"button_press_event"
                        ,GTK_SIGNAL_FUNC(scroll_click),NULL);

	process_clist=gtk_clist_new_with_titles(4,nom_cols);
	gtk_clist_column_titles_active(GTK_CLIST(process_clist));
		
	set_proc_list();
		
	gtk_clist_set_shadow_type (GTK_CLIST(process_clist), GTK_SHADOW_IN);
		
	gtk_clist_set_column_width(GTK_CLIST(process_clist),0,75);
	gtk_clist_set_column_width(GTK_CLIST(process_clist),1,86);
	gtk_clist_set_column_width(GTK_CLIST(process_clist),2,56);
		 
	gtk_container_add(GTK_CONTAINER(scrolled_window),process_clist);
	gtk_widget_show(process_clist);

	gtk_widget_show(scrolled_window);
	return scrolled_window;
}

static gint update_proc_list(gpointer data)
{
	fprintf(stderr, "Update proc list\n");
	if(popup_open==TRUE)
	{
		gtk_clist_freeze(GTK_CLIST(process_clist));
		gtk_clist_clear(GTK_CLIST(process_clist));
		set_proc_list();
		gtk_clist_thaw(GTK_CLIST(process_clist));
	}
	return 1;
}

static void create_popup(void)
{
	gint scrX,scrY,posX,posY;	
	GtkWidget *frame_main;
	GtkWidget *scroll_win;
	GtkWidget *vbox;
	
	if(popup_open==FALSE)
	{
		popup_open=TRUE;
	
		timer_list=gtk_timeout_add(updatetime,update_proc_list,applet);

		popup_window=gtk_window_new(GTK_WINDOW_POPUP);

		gtk_window_set_policy(GTK_WINDOW(popup_window),0,0,1);
		gtk_window_set_title(GTK_WINDOW(popup_window),(gchar *)"Process list");
	
		gdk_window_get_pointer(NULL,&posX,&posY,NULL);

		scrX=gdk_screen_width();
		scrY=gdk_screen_height();
		if(posX+popup_width >= scrX)
			posX-=popup_width;
		if(posY-popup_height+10 <= 0)
			posY+=popup_height+20;

		gtk_widget_set_uposition(popup_window,posX,posY-popup_height-10);
		gtk_widget_set_usize(GTK_WIDGET(popup_window),popup_width,popup_height);
		gtk_widget_show(popup_window);

		scroll_win=create_scroll_win(popup_window);

		gtk_signal_connect(GTK_OBJECT(popup_window),"button_press_event"
							,GTK_SIGNAL_FUNC(close_pop),NULL);


		vbox=gtk_vbox_new(FALSE,1);
     
		frame_main=gtk_frame_new(NULL);
		gtk_frame_set_shadow_type(GTK_FRAME(frame_main),GTK_SHADOW_OUT);
        
		
		gtk_box_pack_start(GTK_BOX(vbox),scroll_win,TRUE,TRUE,0);
		gtk_container_add(GTK_CONTAINER(frame_main),vbox);
		gtk_container_add(GTK_CONTAINER(popup_window),frame_main);
		gtk_widget_show(frame_main);
		
		gtk_widget_show(vbox);

		gtk_signal_connect(GTK_OBJECT(vbox),"button_press_event"
                        	,GTK_SIGNAL_FUNC(scroll_click),NULL);

		
		gdk_pointer_grab(popup_window->window,TRUE,
					     GDK_BUTTON_PRESS_MASK
            	        | GDK_BUTTON_RELEASE_MASK
                	    | GDK_ENTER_NOTIFY_MASK
                    	| GDK_LEAVE_NOTIFY_MASK
	                    | GDK_POINTER_MOTION_MASK,
    	                NULL,NULL,GDK_CURRENT_TIME);
                    
		gtk_grab_add(popup_window);

	}

}

void scan_str(const char *buffer, char *field[], int max) {
	char *buf;
	int i;
	char *token;

	buf = strdup(buffer);
	token = strtok(buf, "\t");

	for(i=0; i < max; i++) {
		token = strtok(NULL, " ");
	if (field[i])
    	free(field[i]);
	field[i]=strdup(token);
	}
	free(buf);
}

void scan_int(const char *buffer, int *field, int max) {
	char *buf;
	int i;
	char *token;

	buf = strdup(buffer);
	token = strtok(buf, "\t");

	for(i=0; i < max; i++) {
		token = strtok(NULL, " ");
		field[i] = atoi(token);
	}
	free(buf);
}

#define IM_BUFSIZE  10 + (22 * ISDN_MAX_CHANNELS)
#define ISDNINFO "/dev/isdninfo"
static ISDN_STATUS *
get_isdn_status()
{
	ISDN_STATUS *isdn_status;
	FILE *isdninfo;
	char buf[IM_BUFSIZE];

	if (!(isdninfo = fopen(ISDNINFO, "r"))) {
		fprintf(stderr, "No such file %s, bye\n", ISDNINFO);
		exit(1);
		applet_widget_gtk_main_quit();
	}

	isdn_status = malloc(sizeof(ISDN_STATUS));
	memset(isdn_status->idmap,0,sizeof(isdn_status->idmap));
	memset(isdn_status->phone,0,sizeof(isdn_status->phone));

	fgets(buf, IM_BUFSIZE, isdninfo);
	scan_str(buf, isdn_status->idmap, ISDN_MAX_CHANNELS);

	fgets(buf, IM_BUFSIZE, isdninfo);
	scan_int(buf, isdn_status->chmap, ISDN_MAX_CHANNELS);

	fgets(buf, IM_BUFSIZE, isdninfo);
	scan_int(buf, isdn_status->drmap, ISDN_MAX_CHANNELS);

	fgets(buf, IM_BUFSIZE, isdninfo);
	scan_int(buf, isdn_status->usage, ISDN_MAX_CHANNELS);

	fgets(buf, IM_BUFSIZE, isdninfo);
	scan_int(buf, isdn_status->flags, ISDN_MAX_DRIVERS);

	fgets(buf, IM_BUFSIZE, isdninfo);
	scan_str(buf, isdn_status->phone, ISDN_MAX_CHANNELS);

	fclose(isdninfo);
                                 
	return(isdn_status);
}
#undef IM_BUFSIZE

static void
redraw_pixmaps(void)
{
	ISDN_STATUS *isdn_status;
	gchar tiptext[200];

	isdn_status = get_isdn_status();
	switch(isdn_status->usage[0] & ISDN_USAGE_MASK) {
		case 0:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap0), none_in_pixmap, none_in_pixmap_mask);
			break;
		case 1:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap0), raw_in_pixmap, raw_in_pixmap_mask);
			break;
		case 2:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap0), modem_in_pixmap, modem_in_pixmap_mask);
			break;
		case 3:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap0), net_in_pixmap, net_in_pixmap_mask);
			break;
		case 4:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap0), voice_in_pixmap, voice_in_pixmap_mask);
			break;
		case 5:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap0), fax_in_pixmap, fax_in_pixmap_mask);
			break;
		default:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap0), none_in_pixmap, none_in_pixmap_mask);
			break;
	}

	switch(isdn_status->usage[1] & ISDN_USAGE_MASK) {
		case 0:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap1), none_in_pixmap, none_in_pixmap_mask);
			break;
		case 2:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap1), modem_in_pixmap, modem_in_pixmap_mask);
			break;
		case 3:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap1), net_in_pixmap, net_in_pixmap_mask);
			break;
		case 4:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap1), voice_in_pixmap, voice_in_pixmap_mask);
			break;
		case 5:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap1), fax_in_pixmap, fax_in_pixmap_mask);
			break;
		default:
			gtk_pixmap_set(GTK_PIXMAP(button_pixmap1), none_in_pixmap, none_in_pixmap_mask);
			break;
	}

	sprintf(tiptext, "line0: %s\nline1: %s", isdn_status->phone[0], isdn_status->phone[1]);
	gtk_tooltips_set_tip(tooltip, applet, tiptext, NULL);
	free(isdn_status);
}

static void
create_pixmaps(void)
{
	GtkStyle *style;
	GdkBitmap *mask;
	style = gtk_widget_get_style(applet);

	green_pixmap = gdk_pixmap_create_from_xpm_d(applet->window, &mask,
	             &style->bg[GTK_STATE_NORMAL], (gchar **)gimon_green);
	yellow_pixmap = gdk_pixmap_create_from_xpm_d(applet->window, &mask,
	             &style->bg[GTK_STATE_NORMAL], (gchar **)gimon_yellow);
	red_pixmap = gdk_pixmap_create_from_xpm_d(applet->window, &mask,
	             &style->bg[GTK_STATE_NORMAL], (gchar **)gimon_red);
}

static GtkWidget
*create_counter(void)
{
	GtkWidget *vbox;
	button = gtk_button_new();
	gtk_signal_connect(GTK_OBJECT(button),"clicked",
	  GTK_SIGNAL_FUNC(create_popup), NULL);
	tooltip=gtk_tooltips_new();
	gtk_tooltips_set_tip (tooltip, applet, "lasjf", NULL);

	create_pixmaps();

	button_pixmap0 = gtk_pixmap_new(green_pixmap, NULL);
	button_pixmap1 = gtk_pixmap_new(green_pixmap, NULL);

	vbox = gtk_hbox_new(FALSE, 4);
	gtk_box_pack_start(GTK_BOX(vbox), button_pixmap0, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), button_pixmap1, TRUE, TRUE, 0);
	gtk_container_add(GTK_CONTAINER(button), vbox);

	redraw_pixmaps();
	gtk_widget_show(button);
	return button;
}

static void
bye(GtkWidget *w, gpointer p)
{
	gnome_config_set_int("gimon/general/updatetime", updatetime);
	gnome_config_set_int("gimon/general/width", popup_width);
	gnome_config_set_int("gimon/general/height", popup_height);
	gnome_config_set_string("gimon/general/none_in_filename", none_in_filename);
	gnome_config_set_string("gimon/general/none_out_filename", none_out_filename);
	gnome_config_set_string("gimon/general/raw_in_filename", raw_in_filename);
	gnome_config_set_string("gimon/general/raw_out_filename", raw_out_filename);
	gnome_config_set_string("gimon/general/modem_in_filename", modem_in_filename);
	gnome_config_set_string("gimon/general/modem_out_filename", modem_out_filename);
	gnome_config_set_string("gimon/general/net_in_filename", net_in_filename);
	gnome_config_set_string("gimon/general/net_out_filename", net_out_filename);
	gnome_config_set_string("gimon/general/voice_in_filename", voice_in_filename);
	gnome_config_set_string("gimon/general/voice_out_filename", voice_out_filename);
	gnome_config_set_string("gimon/general/fax_in_filename", fax_in_filename);
	gnome_config_set_string("gimon/general/fax_out_filename", fax_out_filename);
	gnome_config_sync();
	gnome_config_drop_all();
	applet_widget_gtk_main_quit();
}

static int
save_session(GtkWidget *w, gpointer p)
{
	gnome_config_set_int("gimon/general/updatetime", updatetime);
	gnome_config_set_int("gimon/general/width", popup_width);
	gnome_config_set_int("gimon/general/height", popup_height);
	gnome_config_set_string("gimon/general/none_in_filename", none_in_filename);
	gnome_config_set_string("gimon/general/none_out_filename", none_out_filename);
	gnome_config_set_string("gimon/general/raw_in_filename", raw_in_filename);
	gnome_config_set_string("gimon/general/raw_out_filename", raw_out_filename);
	gnome_config_set_string("gimon/general/modem_in_filename", modem_in_filename);
	gnome_config_set_string("gimon/general/modem_out_filename", modem_out_filename);
	gnome_config_set_string("gimon/general/net_in_filename", net_in_filename);
	gnome_config_set_string("gimon/general/net_out_filename", net_out_filename);
	gnome_config_set_string("gimon/general/voice_in_filename", voice_in_filename);
	gnome_config_set_string("gimon/general/voice_out_filename", voice_out_filename);
	gnome_config_set_string("gimon/general/fax_in_filename", fax_in_filename);
	gnome_config_set_string("gimon/general/fax_out_filename", fax_out_filename);
	gnome_config_sync();
	gnome_config_drop_all();
	return FALSE;
}

int
main(int argc, char *argv[])
{
	/* initialize the i18n stuff */
	textdomain (PACKAGE);
	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
	
	updatetime = 10000;
	
	applet_widget_init("gimon_applet", VERSION, argc, argv, NULL, 0, NULL);
	applet = applet_widget_new("gimon_applet");
	updatetime = gnome_config_get_int("gimon/general/updatetime=10000");
	popup_width = gnome_config_get_int("gimon/general/width=310");
	popup_height = gnome_config_get_int("gimon/general/height=70");

	none_in_filename = gnome_config_get_string("gimon/general/none_in_filename=/usr/local/gnome/share/pixmaps/gimon-red.png");
	loadup_file(&none_in_pixmap, &none_in_pixmap_mask, none_in_filename);
	none_out_filename = gnome_config_get_string("gimon/general/none_out_filename=/usr/local/gnome/share/pixmaps/gimon-red.png");
	loadup_file(&net_out_pixmap, &none_out_pixmap_mask, none_out_filename);

	net_in_filename = gnome_config_get_string("gimon/general/net_in_filename=/usr/local/gnome/share/pixmaps/gimon-green.png");
	loadup_file(&net_in_pixmap, &net_in_pixmap_mask, net_in_filename);
	net_out_filename = gnome_config_get_string("gimon/general/net_out_filename=/usr/local/gnome/share/pixmaps/gimon-green.png");
	loadup_file(&net_out_pixmap, &net_out_pixmap_mask, net_out_filename);

	modem_in_filename = gnome_config_get_string("gimon/general/modem_in_filename=/usr/local/gnome/share/pixmaps/gimon-green.png");
	loadup_file(&modem_in_pixmap, &modem_in_pixmap_mask, modem_in_filename);
	modem_out_filename = gnome_config_get_string("gimon/general/modem_out_filename=/usr/local/gnome/share/pixmaps/gimon-green.png");
	loadup_file(&modem_out_pixmap, &modem_out_pixmap_mask, modem_out_filename);

	voice_in_filename = gnome_config_get_string("gimon/general/voice_in_filename=/usr/local/gnome/share/pixmaps/gimon-green.png");
	loadup_file(&voice_in_pixmap, &voice_in_pixmap_mask, voice_in_filename);
	voice_out_filename = gnome_config_get_string("gimon/general/voice_out_filename=/usr/local/gnome/share/pixmaps/gimon-green.png");
	loadup_file(&voice_out_pixmap, &voice_out_pixmap_mask, voice_out_filename);

	fax_in_filename = gnome_config_get_string("gimon/general/fax_in_filename=/usr/local/gnome/share/pixmaps/gimon-green.png");
	loadup_file(&fax_in_pixmap, &fax_in_pixmap_mask, fax_in_filename);
	fax_out_filename = gnome_config_get_string("gimon/general/fax_out_filename=/usr/local/gnome/share/pixmaps/gimon-green.png");
	loadup_file(&fax_out_pixmap, &fax_out_pixmap_mask, fax_out_filename);

	raw_in_filename = gnome_config_get_string("gimon/general/raw_in_filename=/usr/local/gnome/share/pixmaps/gimon-green.png");
	loadup_file(&raw_in_pixmap, &raw_in_pixmap_mask, raw_in_filename);
	raw_out_filename = gnome_config_get_string("gimon/general/raw_out_filename=/usr/local/gnome/share/pixmaps/gimon-green.png");
	loadup_file(&raw_out_pixmap, &raw_out_pixmap_mask, raw_out_filename);


	if(!applet)
		g_error("Cannot create gimon applet!");

	gtk_widget_realize(applet);
	counter = create_counter();
	applet_widget_add(APPLET_WIDGET(applet), counter);
	gtk_widget_show_all(applet);

	gtk_signal_connect(GTK_OBJECT(applet), "delete_event",
			   GTK_SIGNAL_FUNC(bye), NULL);
	gtk_signal_connect(GTK_OBJECT(applet), "save_session",
			   GTK_SIGNAL_FUNC(save_session), NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
					      "about",                    
					      GNOME_STOCK_MENU_ABOUT,
					      _("About"),         
					      GTK_SIGNAL_FUNC(about_cb),
					      NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
					      "preference",                    
					      GNOME_STOCK_MENU_PREF,
					      _("Preferences"),         
					      GTK_SIGNAL_FUNC(prefs_cb),
					      NULL);

	count_timer(NULL);
	timeout = gtk_timeout_add(updatetime, (GtkFunction)count_timer, NULL);
	
	applet_widget_gtk_main();
	return(0);
}
